﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ZerimThing
{
    using System;
using System.Collections.Generic;
    using System.Diagnostics;


    internal class clsProperty
    {
        internal readonly   string          Name;        

        internal readonly   List<string>    ArgNames       = new List<string>();
        internal readonly   List<Type>      ArgTypes       = new List<Type>();
        internal readonly   List<Object>    ArgDefaults    = new List<Object>();

        private List<Object> ArgValues = new List<Object>();

        // copy constructor; i.e., make this property an exact copy of a different property.
        // this is used so you don't modify the existing properties list when you're in frmProperties or frmAddProperties
        // until you hit OK.
        internal clsProperty(clsProperty iCopyFrom)
        {
            this.Name = iCopyFrom.Name;
            
            this.ArgNames = new List<string>(iCopyFrom.ArgNames);
            this.ArgTypes = new List<Type>(iCopyFrom.ArgTypes);
            this.ArgDefaults = new List<Object>(iCopyFrom.ArgDefaults);

        }

        internal clsProperty(string iName, List<string> iArgNames, List<Type> iArgTypes, List<Object> iArgDefaults)
        {
            if (string.IsNullOrWhiteSpace(iName))
            {
                throw new ArgumentException("Property must have a valid name.");
            }

            this.Name = iName;

            bool tParameterLengthsOK =  (
                                            (iArgNames.Count == iArgTypes.Count) && 
                                            (iArgTypes.Count == iArgDefaults.Count)
                                        );

            if (!tParameterLengthsOK)
            {
                throw new ArgumentException("Each argument must have a name, type, and default value.");
            }

            for (int tArgumentIndex = 0; (tArgumentIndex < iArgNames.Count); tArgumentIndex++)
            {
                string  tArgName        = iArgNames[tArgumentIndex];
                Type    tArgType        = iArgTypes[tArgumentIndex];
                object  tDefaultValue   = iArgDefaults[tArgumentIndex];

                if(tDefaultValue == null)
                {
                    throw new ArgumentException("Argument " + tArgumentIndex + ": no default value.");
                }

                if (String.IsNullOrWhiteSpace(tArgName))
                {
                    throw new ArgumentException("Argument " + tArgumentIndex + ": name must not be blank.");
                }

                if (!IsValidArgType(tArgType))
                {
                    throw new ArgumentException("Argument " + tArgumentIndex + ": type " + tArgType.ToString() + " is not supported.");
                }

                this.ArgNames.Add(tArgName);
                this.ArgTypes.Add(tArgType);
                this.ArgDefaults.Add(tDefaultValue);
                this.ArgValues.Add(tDefaultValue);
            }
        }

        internal int GetArgumentCount()
        {
            return this.ArgValues.Count();
        }



        internal string GetArgumentValueString(int iArgIndex)
        {
            string qArgValueString = null;

            if ((iArgIndex < 0) || (iArgIndex > (ArgValues.Count - 1)))
            {
                throw new IndexOutOfRangeException("Argument index " + iArgIndex + " is invalid, arguments list has " + ArgValues.Count + " items.");
            }

            Object tArgValue = this.ArgValues[iArgIndex];

            // this is a warning to us.... in a debug build it will stop right now
            // I'm asserting because this class shouldn't allow null values to even get this far.
            Debug.Assert(tArgValue != null, "Null argument value found in clsProperty");

            
            // ....but we still have to recover from it for release builds....
            if(tArgValue == null)
            {
                qArgValueString = "";
            }
            else
            {
                qArgValueString = tArgValue.ToString();
            }


            return qArgValueString;
        }

        internal string GetArgumentName(int iArgIndex)
        {
            string qArgumentName = null;

            if ((iArgIndex < 0) || (iArgIndex > (ArgValues.Count - 1)))
            {
                throw new IndexOutOfRangeException("Argument index " + iArgIndex + " is invalid, arguments list has " + ArgValues.Count + " items.");
            }

            qArgumentName = this.ArgNames[iArgIndex];

            return qArgumentName;

        }

        internal void SetArgumentValue(int iArgIndex, string iValueString)
        {          

            if ((iArgIndex < 0) || (iArgIndex > (ArgValues.Count - 1)))
            {
                throw new IndexOutOfRangeException("Argument index " + iArgIndex + " is invalid, arguments list has " + ArgValues.Count + " items.");
            }

            Object  tValue      = null;
            Type    tArgType    = this.ArgTypes[iArgIndex];

            if (TryParseArgValue(iValueString, tArgType, out tValue))
            {
                this.ArgValues[iArgIndex] = tValue;
            }
            else
            {
                throw new ArgumentException("Could not parse string " + iValueString + " as argument type " + tArgType.ToString());
            }
        }



        // for the sake of this example assume that properties can only be one of these types.
        internal static bool IsValidArgType(Type iType)
        {
            return
                (
                    (iType == typeof(System.String))  ||
                    (iType == typeof(System.Int32))     ||
                    (iType == typeof(System.Int64))    ||
                    (iType == typeof(System.Single))   ||
                    (iType == typeof(System.Double))
                );          


        }

        private static bool TryParseArgValue(string iValueString, Type iParseAsType, out object qValue)
        {
            qValue = null;

            if (iParseAsType == typeof(System.String))
            {
                qValue = iValueString;
                return true;
            }
            else if (iParseAsType == typeof(System.Int32))
            {
                int tParsedInt;
                if (int.TryParse(iValueString, out tParsedInt))
                {
                    qValue = tParsedInt;
                    return true;
                }
                else
                {
                    return false;
                }

            }
            else if (iParseAsType == typeof(System.Int64))
            {
                long tParsedLong;
                if (long.TryParse(iValueString, out tParsedLong))
                {
                    qValue = tParsedLong;
                    return true;
                }
                else
                {
                    return false;
                }
            }

            else if (iParseAsType == typeof(System.Single))
            {
                float tParsedFloat;
                if (float.TryParse(iValueString, out tParsedFloat))
                {
                    qValue = tParsedFloat;
                    return true;
                }
                else
                {
                    return false;
                }
            }

            else if (iParseAsType == typeof(System.Double))
            {
                double tParsedDouble;
                if (double.TryParse(iValueString, out tParsedDouble))
                {
                    qValue = tParsedDouble;
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }

        }
    };

}
